/*******************************************************************************
 * Baseline Economy simulation based on Lengnick, M. (2013). Agent-based
 * macroeconomics: A baseline model. Journal of Economic Behavior &
 * Organization. 86:102-120. DOI: https://doi.org/10.1016/j.jebo.2012.12.021
 * 
 * @copyright Copyright 2018 Brandenburg University of Technology, Germany.
 * @license The MIT License (MIT)
 * @author Luis Gustavo Nardin
 * @author Gerd Wagner
 ******************************************************************************/
/*******************************************************************************
 * Simulation Parameters
 ******************************************************************************/
// sim.scenario.title = "..."; // optional
// sim.scenario.name = "..."; // optional
// sim.scenario.shortDescription = "<p></p>"; // optional
// sim.scenario.creator = "..."; // optional
// sim.scenario.created = "..."; // optional
// sim.scenario.modified = "..."; // optional
sim.scenario.simulationEndTime = 4000;
sim.scenario.idCounter = 0; // optional
// sim.scenario.randomSeed = 1234; // optional
/*******************************************************************************
 * Simulation Configuration
 ******************************************************************************/
sim.config.createLog = false;
sim.config.visualize = false;
sim.config.userInteractive = false;
/*******************************************************************************
 * Simulation Model
 ******************************************************************************/
sim.model.time = "discrete";
sim.model.timeUnit = "D"; // days

sim.model.objectTypes = ["Firm", "Household"];
sim.model.eventTypes = ["StartOfMonth", "EachDay", "EndOfMonth"];

// Global variables
sim.model.v.verbose = false; // Enable/Disable console messages
sim.model.v.month = 1; // Current month
sim.model.v.day = 1; // Current day
sim.model.v.daysMonth = 21; // Number of consecutive days in a month
sim.model.v.numF = 10; // Number of Firms
sim.model.v.numH = 100; // Number of Households
sim.model.v.n = 7; // Number of Firms per Household
sim.model.v.money = 98.78; // Initial money amount

/*******************************************************************************
 * Define Initial State
 ******************************************************************************/
// Objects
sim.scenario.initialState.objects = {};

// Events
sim.scenario.initialState.events = [{
  typeName: "StartOfMonth",
  occTime: 1,
  priority: 2
}, {
  typeName: "EachDay",
  occTime: 1,
  priority: 1
}, {
  typeName: "EndOfMonth",
  occTime: sim.model.v.daysMonth + 1,
  priority: 0
}];

// Setup the initial state of the simulation scenario
sim.scenario.setupInitialState =
    function () {
      var i, j, iFirms, kFirms, firms, firm, hh, eFirms, eFirm;
      
      // Create Firms
      for ( i = 1; i <= sim.model.v.numF; i += 1 ) {
        firm = new Firm( {
          id: i,
          typeName: "Firm",
          name: "F" + i,
          money: 0,
          reserve: 0,
          inventoryLevel: 50,
          consumptionGoodPrice: 1 * ((rand.uniform( 0, 1 ) - 0.5) / 50 + 1),
          demand: 0,
          workers: [],
          wageRate: 52 * ((rand.uniform( 0, 1 ) - 0.5) / 50 + 1),
          openPosition: false,
          filledPosition: false,
          lastOpenPosition: 0,
          closePosition: false,
          startPlanning: 24 + rand.uniformInt( 0, 84 )
        } );
        
        sim.addObject( firm );
      }
      
      iFirms = cLASS["Firm"].instances;
      kFirms = Object.keys( iFirms );
      eFirms = Object.keys( iFirms );
      for ( i = 1; i <= sim.model.v.numH; i += 1 ) {
        
        // Randomly select n Firms the Household will be able to buy products
        // from
        firms = [];
        util.shuffleArray( kFirms );
        j = 0;
        do {
          firm = iFirms[kFirms[j]];
          if ( !(firm in firms) ) {
            firms.push( firm );
            j += 1;
          }
        } while ( j < Math.min( sim.model.v.n, kFirms.length ) );
        
        // Select employer guaranteeing one worker per firm
        if ( i <= sim.model.v.numF ) {
          eFirm =
              iFirms[eFirms.splice( rand.uniformInt( 0, eFirms.length - 1 ), 1 )[0]];
        } else {
          eFirm = iFirms[kFirms[rand.uniformInt( 0, kFirms.length - 1 )]];
        }
        
        // Create Households
        hh = new Household( {
          id: sim.model.v.numF + i,
          typeName: "Household",
          name: "HH" + i,
          employed: true,
          employer: eFirm,
          reservationWage: 0,
          receivedWage: 0,
          money: sim.modelVariables.money,
          dailyDemand: 0,
          preferredSuppliers: firms.slice(),
          blacklistedSuppliers: {}
        } );
        
        sim.addObject( hh );
        
        // Add household into the employer's working list
        eFirm.workers.push( hh );
      }
    };

/*******************************************************************************
 * Define Experiment
 ******************************************************************************/
// sim.experiment.parameters = [{
// name: "n",
// values: [4, 5, 6, 7]
// }, {
// name: "money",
// values: [98.78, 1.00, 1000.00]
// }];
// sim.experiment.replications = 5;
// sim.experiment.seeds = [7836, 2893, 6235, 3827, 1432]
/*******************************************************************************
 * Define Output Statistics Variables
 ******************************************************************************/
sim.model.statistics = {
  // "money": {
  // range: "PositiveDecimal",
  // label: "Average Firms liquidity",
  // initialValue: 0,
  // showTimeSeries: true,
  // computeOnlyAtEnd: false,
  // expression: function () {
  // var total = 0;
  // var firms = cLASS["Firm"].instances;
  // Object.keys( firms ).forEach( function ( objId ) {
  // total += firms[objId].money;
  // } );
  //      
  // return total / Object.keys( firms ).length;
  // }
  // },
  // "inventoryLevel": {
  // range: "PositiveDecimal",
  // label: "Average Inventory",
  // initialValue: 0,
  // showTimeSeries: true,
  // computeOnlyAtEnd: false,
  // expression: function () {
  // var total = 0;
  // var firms = cLASS["Firm"].instances;
  // Object.keys( firms ).forEach( function ( objId ) {
  // total += firms[objId].inventoryLevel;
  // } );
  //      
  // return total / Object.keys( firms ).length;
  // }
  // },
  // "workers": {
  // range: "PositiveDecimal",
  // label: "Average number of workers",
  // initialValue: 0,
  // showTimeSeries: true,
  // computeOnlyAtEnd: false,
  // expression: function () {
  // var total = 0;
  // var firms = cLASS["Firm"].instances;
  // Object.keys( firms ).forEach( function ( objId ) {
  // total += firms[objId].workers.length;
  // } );
  //      
  // return total / Object.keys( firms ).length;
  // }
  // },
  "consumptionGoodPrice": {
    range: "PositiveDecimal",
    label: "Average Price",
    initialValue: 0,
    showTimeSeries: true,
    computeOnlyAtEnd: false,
    expression: function () {
      var total = 0;
      var firms = cLASS["Firm"].instances;
      Object.keys( firms ).forEach( function ( objId ) {
        total += firms[objId].consumptionGoodPrice;
      } );
      
      return total / Object.keys( firms ).length;
    }
  },
  "employment": {
    range: "PositiveDecimal",
    label: "% Employment",
    initialValue: 0,
    showTimeSeries: true,
    computeOnlyAtEnd: false,
    expression: function () {
      var total = 0;
      var hhs = cLASS["Household"].instances;
      Object.keys( hhs ).forEach( function ( objId ) {
        if ( hhs[objId].employed ) {
          total += 1;
        }
      } );
      
      return total / Object.keys( hhs ).length;
    }
  }
};
